--
-- Fuel Useability for motorized vehicles Specialization
-- by Blacky_BPG
--

fuelAdjust = {};
fuelAdjust.modDir = g_currentModDirectory;
local misha = 0;
local misdiff = 0;
local lastha = 0;
local lastfac = 0;
local lastdiff = 0;
local lasttime = 0;

function fuelAdjust.prerequisitesPresent(specializations)
	return true;
end;
function fuelAdjust:load(xmlFile)
	self.fuelAdjust = {};
	self.fa = {};
	self.fa.fuelUsage = self.fuelUsage;
	self.fuelUsage = 0;  -- Override for recalculation
	self.fa.backupRpmMin = 500;
	self.fa.backupRpmMax = 1000;
	self.fa.rpmLimiterFix = 0;
	self.needFuel = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.needFuel"), true);
	if self.motor ~= nil then
		if self.motor.minRpm ~= nil then
			self.fa.backupRpmMin = self.motor.minRpm;
			self.saveMinRpm = self.motor.minRpm;  	-- PloughingSpec Value
		end;
		for i=1,table.getn(self.motor.maxRpm) do
			if self.fa.backupRpmMax < self.motor.maxRpm[i] then
				self.fa.backupRpmMax = self.motor.maxRpm[i];
			end;
		end;
	end;
	self.fa.lastFuelFill = self.fuelFillLevel;

	self.addfuelAdjust = SpecializationUtil.callSpecializationsFunction("addfuelAdjust");

	self.fa.fuelUsed = 0;
	self.fa.fuelUsageSum = 0;
	self.fa.fuelUsageSumTmp = 0;
	self.fa.counter = 0;
	self.fa.OverallFuelUsage = 0;
	self.fa.OverallFuel = 0;
	self.fa.OverallFuelUsageFactor = 0;
	self.fa.mode1 = false;
	self.fa.mode2 = false;
	self.fa.fixStartFuel = -1;
	self.mtrlmrpm = 0;
	if g_currentMission.extraHudMode == nil then
		g_currentMission.extraHudMode = 0;
	end;
	self.fa.realisticFuelFactor = 1;
	if self.isRealistic then
		self.fa.realisticFuelFactor = 2;
		self.fa.fuelUsage = (self.realBaseMaxFuelUsage / 10000);
		self.realMaxFuelUsage = 0;  -- Override for recalculation
		self.realBaseMaxFuelUsage = 0;  -- Override for recalculation
		self.fa.backupRpmMin = 100;
		self.saveMinRpm = 100;
		self.fa.backupRpmMax = 3200 -- self.realMaxVehicleSpeed * 55;
		if self.motor == nil then
			self.motor = {};
			self.motor.lastMotorRpm = 0;
		end;
	end;
	self.fa.fuelUsage = self.fa.fuelUsage / self.fa.realisticFuelFactor;
	self.fa.fuelUsageBackup = self.fa.fuelUsage;
	self.fa.idleUsage = (self.fa.fuelUsage / self.fa.backupRpmMax) * self.fa.backupRpmMin;
	if g_currentMission.extraHudMode == nil then
		g_currentMission.extraHudMode = 0;
	end;
end;

function fuelAdjust:round(val, decimal)
	if decimal then
		return math.floor((val*10^decimal)+0.5)/(10^decimal);
	else
		return math.floor(val+0.5);
	end;
end;

function fuelAdjust:update(dt)
	if self.isMotorStarted then
		if self.isRealistic then
			if self.realMotorLoadS ~= nil then
				self.mtrlmrpm = math.max(math.tan(self.realMotorLoadS) * 1900,self.fa.backupRpmMin);
			else
				self.mtrlmrpm = 100;
			end;
		else
			self.mtrlmrpm = Utils.getNoNil(self.motor.lastMotorRpm,0);
		end;
		if self.fuelFillLevel ~= nil and self.fuelFillLevel <= 0.1 then
			if self.isHired then
				self:dismiss();
			end;
			if self.setManualIgnitionMode ~= nil then
				if self.ignitionMode == 1 or self.ignitionMode == 2 then
					self:setManualIgnitionMode(3);
				end;
			else
				if self.stopMotor ~= nil then
					self:stopMotor(true);
				end;
			end;
		end;
	else
		self.mtrlmrpm = 0;
		if self.stopMotor ~= nil and not self.isMotorStarted then
			if self.isControlled or self:getIsActive() then
				if self.setManualIgnitionMode == nil then
					self:startMotor(true);
				end;
			end;
		end;
		if self.fa ~= nil and self.fa.fixStartFuel ~= nil and self.fa.fixStartFuel > -1 then
			if self.fuelFillLevel ~= nil and self.fuelFillLevel > self.fa.fixStartFuel then
				local diff = self.fuelFillLevel - self.fa.fixStartFuel;
				local delta = diff * g_fuelPricePerLiter;
				g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal - delta;
				g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession - delta;
				g_currentMission:addSharedMoney(delta, "vehicleRunningCost");
				self:setFuelFillLevel(self.fa.fixStartFuel);
				self.fa.fixStartFuel = -1;
			end;
		end;
	end;
	if self.isFuelFilling then
		self.fa.lastFuelFill = self.fuelFillLevel;
	end;
	if self.needFuel and (self:getIsActive() or self:getIsHired()) then
	  if self.isMotorStarted then
		self.fa.fuelUsed = 0;
		self.fuelUsage = 0;  -- Override for recalculation
		self.realMaxFuelUsage = 0;  -- Override for recalculation
		self.realBaseMaxFuelUsage = 0;  -- Override for recalculation
		if self.damageLevel ~= nil then
			self.fa.fuelUsage = self.fa.fuelUsageBackup + (self.fa.fuelUsageBackup * (self.damageLevel * (0.0066 / (self.fa.realisticFuelFactor))));
		end;
		if self.fa.backupRpmMin ~= self.motor.minRpm then
			self.saveMinRpm = self.fa.backupRpmMin;  	-- PloughingSpec Value Override
			self.fa.fuelUsed = (((self.fa.fuelUsage / self.fa.backupRpmMax) * self.fa.backupRpmMin) - self.fa.idleUsage) * (0.66 / (self.fa.realisticFuelFactor));
		else
			self.fa.fuelUsed = 0;
		end;

		self.fa.fuelUsed = (((self.fa.fuelUsage / self.fa.backupRpmMax) * math.max(0, self.mtrlmrpm)) - self.fa.idleUsage);

		self.fa.mode1 = false;
		self.fa.mode2 = false;
		if self.movingDirection ~= 0 then
			local fLevel, capacity = self:getAttachedTrailersFillLevelAndCapacity();
			if fLevel ~= nil then
				if fLevel > 0 then
					self.fa.fuelUsed = self.fa.fuelUsed + (fLevel * (0.00000001 / (self.fa.realisticFuelFactor)));
					self.fa.mode1 = true;
				end;
			end;
			fLevel = 0;
			if self.fillLevel ~= nil then
				self.fa.fuelUsed = self.fa.fuelUsed + (self.fillLevel * (0.00000001 / (self.fa.realisticFuelFactor)));
				self.fa.mode1 = true;
			end;
			if self.AllradV4Active ~= nil then
				if self.AllradV4Active == true then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.0425 / (self.fa.realisticFuelFactor)));
				end;
			elseif self.AllradV41Active ~= nil then
				if self.AllradV41Active == true then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.0425 / (self.fa.realisticFuelFactor)));
				end;
			elseif self.AllWheelDriveActive ~= nil then
				if self.AllWheelDriveActive == true then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.0425 / (self.fa.realisticFuelFactor)));
				end;
			elseif self.realAWDModePossible then
				if self.realAWDModeOn then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.0425 / (self.fa.realisticFuelFactor)));
				end;
			end;
		end;

		for i=1, table.getn(self.attachedImplements) do
			local implement = self.attachedImplements[i].object;
			if implement.isTurnedOn == true then
				self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.03 / (self.fa.realisticFuelFactor)));
			end;
			if self.movingDirection ~= 0 then
				for x = 1, table.maxn(implement.specializations) do
					local vs = implement.specializations[x];
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("cultivator") then
						if implement.cultivatorHasGroundContact == true then 					-- or implement:isLowered() then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.045 / (self.fa.realisticFuelFactor)));
							self.fa.mode2 = true;
						end;
					end;
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("plough") then
						if implement.ploughHasGroundContact == true then						-- or implement:isLowered() then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.065 / (self.fa.realisticFuelFactor)));
							self.fa.mode2 = true;
						end;
					end;
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("sowingMachine") then
						if implement.sowingMachineHasGroundContact == true then				-- or implement:isLowered() then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.035 / (self.fa.realisticFuelFactor)));
							self.fa.mode2 = true;
						end;
					end;
				end;
			end;
		end;

		if self.isThreshing or self.isAIThreshing then
			self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * (0.03 / (self.fa.realisticFuelFactor^2)));
		end;

		self.fa.fuelUsed = (self.fa.fuelUsed / 1.33) + self.fa.idleUsage;
		if not self.isRealistic then
			self.fa.fuelUsed = self.fa.fuelUsed / 3;
		end;
		if self.fuelFillLevel > 0 then
			if self.fa.counter >= 30 then
				self.fa.fuelUsageSum = (self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * self.fa.realisticFuelFactor)) / (self.fa.counter + 1);
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSum;
				self.fa.OverallFuelUsageFactor = self.fa.OverallFuelUsageFactor + 1;
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage * (self.fa.OverallFuelUsageFactor - 1)) + (self.fa.fuelUsageSum * 10000)) / self.fa.OverallFuelUsageFactor;
				self.fa.counter = 0;
			else
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * self.fa.realisticFuelFactor);
				self.fa.counter = self.fa.counter + 1;
			end;
			self.fa.lastFuelFill = self.fa.lastFuelFill - self.fa.fuelUsed;
			if self.isServer then
				self:setFuelFillLevel(self.fa.lastFuelFill);
				g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + self.fa.fuelUsed;
				g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + self.fa.fuelUsed;
			end;
			self.fa.OverallFuel = self.fa.OverallFuel + self.fa.fuelUsed;
		else
			isRun = false;
			if self.stopAITractor~= nil then
				self:stopAITractor();
				self.wasActivated = true;
			elseif self.stopAIThreshing~= nil then
				self:stopAIThreshing();
				self.wasActivated = true;
			end;
		end;
	  else
		if self.fa then
			if self.fa.fuelUsageSum > 0 then
				self.fa.fuelUsageSum = (self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * self.fa.realisticFuelFactor)) / (self.fa.counter + 1);
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSum;
				self.fa.OverallFuelUsageFactor = self.fa.OverallFuelUsageFactor + 1;
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage * (self.fa.OverallFuelUsageFactor - 1)) + (self.fa.fuelUsageSum * 10000)) / self.fa.OverallFuelUsageFactor;
				self.fa.counter = 0;
				self.fa.lastFuelFill = self.fa.lastFuelFill - self.fa.fuelUsed;
				if self.isServer then
					self:setFuelFillLevel(self.fa.lastFuelFill);
					g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + self.fa.fuelUsed;
					g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + self.fa.fuelUsed;
				end;
				self.fa.OverallFuel = self.fa.OverallFuel + self.fa.fuelUsed;
			end;
			self.fa.fuelUsageSum = 0;
		end;
	  end;
	end;
end;
function fuelAdjust:updateTick(dt)
	if lasttime == 0 then
		lasttime = math.random(180,220);
		if self.isMotorStarted then
			self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel);
			if self.fa.OverallFuelUsageFactor > 2000 then
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage / self.fa.OverallFuelUsageFactor) * 1500);
				self.fa.OverallFuelUsageFactor = 1500;
			end;
		end;
	else
		lasttime = lasttime - 1;
	end;
end;
function fuelAdjust:readStream(streamId, connection)
	self.fa.OverallFuelUsage = streamReadFloat32(streamId);
	self.fa.OverallFuelUsageFactor = streamReadInt32(streamId);
	self.fa.OverallFuel = streamReadFloat32(streamId);
end;

function fuelAdjust:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.fa.OverallFuelUsage);
	streamWriteInt32(streamId, self.fa.OverallFuelUsageFactor);
	streamWriteFloat32(streamId, self.fa.OverallFuel);
end;
function fuelAdjust:addfuelAdjust(addOFU,addOFUF, addFU, noEventSend)
	addOFU = math.max(Utils.getNoNil(addOFU, 0.0), 0.0);
	addOFUF = math.max(Utils.getNoNil(addOFUF, 0.0), 0.0);
	addFU = math.max(Utils.getNoNil(addFU, 0.0), 0.0);

	fuelAdjustEvent.sendEvent(self, addOFU, addOFUF, addFU, noEventSend);

	self.fa.OverallFuelUsage = addOFU;
	self.fa.OverallFuelUsageFactor = addOFUF;
	self.fa.OverallFuel = addFU;
end;
function fuelAdjust:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local addfusage = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsage"), self.fa.OverallFuelUsage);
	local addfusagefactor = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsageFactor"), self.fa.OverallFuelUsageFactor);
	local addfused = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsed"), self.fa.OverallFuel);
	local ffl = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#fuelFillLevel"), self.fa.lastFuelFill);
	self.fa.OverallFuelUsage = addfusage;
	self.fa.OverallFuelUsageFactor = addfusagefactor;
	self.fa.OverallFuel = addfused;
	self.fa.lastFuelFill = ffl;
	self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel);
	local fuelFillLevel = getXMLFloat(xmlFile, key.."#fuelFillLevel");
	self.fa.fixStartFuel = fuelFillLevel;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function fuelAdjust:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'OverallFuelUsage="' .. tostring(self.fa.OverallFuelUsage) ..'" OverallFuelUsageFactor="' .. tostring(self.fa.OverallFuelUsageFactor) ..'" OverallFuelUsed="' ..tostring(self.fa.OverallFuel) ..'"';
	return attributes, nil;
end;

function fuelAdjust:onEnter()
end;

function fuelAdjust:onLeave()
end;

function fuelAdjust:draw()
	if self:getIsActive() then
		if SpecializationUtil.hasSpecialization(Motorized, self.specializations) then
			if g_currentMission.extraHudMode < 2 then
				if self.needFuel then
					Motorized.fAOverlay:render();
					setTextBold(true);
					setTextAlignment(RenderText.ALIGN_RIGHT);
					setTextColor(1,1,1,1);
					renderText(0.6530, 0.1015, 0.0195, string.format("%.2f", self.fa.fuelUsageSum * 10000 * 1.166666666) .. "L");

					setTextBold(true);
					setTextAlignment(RenderText.ALIGN_RIGHT);
					setTextColor(1,1,1,1);
					renderText(0.7520, 0.1015, 0.0195, string.format("%.2f", self.fa.OverallFuelUsage) .. "L");
				end;
			end;
		end;
	end;
end;

function fuelAdjust:delete()
end;

function fuelAdjust:mouseEvent(posX, posY, isDown, isUp, button)
end;

function fuelAdjust:keyEvent(unicode, sym, modifier, isDown)
end;

Motorized.update = Utils.appendedFunction(fuelAdjust.update, Motorized.update);

-----------------------------------------------------------------------

fuelAdjustEvent = {};
fuelAdjustEvent_mt = Class(fuelAdjustEvent, Event);

InitEventClass(fuelAdjustEvent, "fuelAdjustEvent");

function fuelAdjustEvent:emptyNew()
    local self = Event:new(fuelAdjustEvent_mt);
    self.className="fuelAdjustEvent";
    return self;
end;

function fuelAdjustEvent:new(vehicle, addOFU, addOFUF, addFU)
	local self = fuelAdjustEvent:emptyNew()
	self.vehicle = vehicle;
	self.OverallFuelUsage = addOFU;
	self.OverallFuelUsageFactor = addOFUF;
	self.OverallFuel = addFU;
	return self;
end;

function fuelAdjustEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.OverallFuelUsage = streamReadFloat32(streamId);
	self.OverallFuelUsageFactor = streamReadInt32(streamId);
	self.OverallFuel = streamReadFloat32(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function fuelAdjustEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteFloat32(streamId, self.OverallFuelUsage);
	streamWriteInt32(streamId, self.OverallFuelUsageFactor);
	streamWriteFloat32(streamId, self.OverallFuel);
end;

function fuelAdjustEvent:run(connection)
	if self.vehicle ~= nil then
		if not connection:getIsServer() then
			g_server:broadcastEvent(fuelAdjustEvent:new(self.vehicle, self.OverallFuelUsage, self.OverallFuelUsageFactor, self.OverallFuel), nil, connection, self.vehicle);
		end;
		self.vehicle:addfuelAdjust(self.OverallFuelUsage, self.OverallFuelUsageFactor, self.OverallFuel, true);
	end;
end;

function fuelAdjustEvent.sendEvent(vehicle, OFU, OFUF, FU, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(fuelAdjustEvent:new(vehicle, OFU, OFUF, FU), nil, nil, vehicle);
		end;
	end;
end;
